#!/usr/bin/env sh

#
# Copyright(c) 2006-2016 ThreatSTOP, Inc. All Rights Reserved
#
# NOTICE: All information contained herein is, and remains the property
# of ThreatSTOP, Inc. and its suppliers, if any. The intellectual and 
# technical concepts contained herein are proprietary to ThreatSTOP, Inc. 
# and its suppliers and certain aspects thereof are protected by United 
# States Patent No. 8,533,822 and United States Patent No. 8,869,237, 
# and are protected by trade secret or copyright law.
#
# US Government Entities: The ThreatSTOP service, software and
# documentation, as applicable, are "commercial computer software" and
# "commercial computer software documentation" developed exclusively at
# private expense by Threatstop, Inc. ("Threatstop"). Pursuant to FAR
# 12.212 or DFARS 227 7202 and their successors, as applicable, use,
# reproduction and disclosure of the software and documentation is
# governed by the terms of Threatstop's commercial agreement.
#

#
# ThreatSTOP OpenBSD PF List Update Script
# These scripts get your ThreatSTOP block list and creates files that PF can
# use as tables. Please see the README.txt file for details on running the 
# scripts.
#
###############################################################################


PREFIX="/usr/local"
CONFIG="$PREFIX/etc/threatstop.conf"
SED_PTR="$PREFIX/sbin/ptr.sed"
SED_APL="$PREFIX/sbin/apl.sed"
VERSION=$(cat $PREFIX/etc/threatstop.VERSION)
LINE="============================================================"

echo "Starting ThreatSTOP v$VERSION update on `date`" ; echo
START=`date +%R:%S`
START_EPOCH=`date +%s`

# Check for and read the config file, abort if not found
if [ ! -f $CONFIG ]
then
	echo "CRITICAL: Cannot read config file: \"$CONFIG\""
	echo "Exiting"
	exit 1
else
	. $CONFIG
fi

# Create temporary files. These are deleted at the end of the script
BLOCK_TMP=`mktemp /tmp/tsblocktmp.XXXXXXXXXX`
ALLOW_TMP=`mktemp /tmp/tsallowtmp.XXXXXXXXXX`

LISTS="$ALLOW_LIST $BLOCK_LIST" 

# Check the availability of the ThreatSTOP DNS servers. Use the first one
# that works, abort if none of them work.
for DNS in $DNS_SERVERS
do
	RUN=`dig @$DNS basic.threatstop.local PTR | sed -n -f $SED_PTR |wc -l`
	if [ $RUN -gt 0 ]
	then
		OKDNSLIST="$OKDNSLIST $DNS"
		#echo "DNS Server $DNS worked"
	else
		echo "DNS Server $DNS failed"
	fi
done
if [ `echo $OKDNSLIST | wc -w` -eq 0 ]
then
	echo "No ThreatSTOP DNS servers are reachable. Aborting"
	exit 1
fi
set $OKDNSLIST
DNS=$1
echo "Using DNS Server $DNS"


# Do a PTR lookup on the main list to get the full complement of 
# block and/or allow lists. Save to temporary files.
DATE=`date`
for LIST in $LISTS
do
	if [[ "$LIST" == *-netb\.* ]]
	then
		LISTTYPE='block'
		TMPFILE=$BLOCK_TMP
	elif [[ "$LIST" == basic\.threatstop\.local ]]
	then
		LISTTYPE='block'
		TMPFILE=$BLOCK_TMP
	elif [[ "$LIST" == *-neta\.* ]]
	then
		LISTTYPE='allow'
		TMPFILE=$ALLOW_TMP
	else
		echo "ERROR: No lists found. Exiting"
		exit 1
	fi

	echo "# $LISTTYPE list $LIST generated on $DATE" >> $TMPFILE
	for PTR in `dig @$DNS $LIST PTR | sed -n -f $SED_PTR`
	do
		echo "Processing $LISTTYPE list $PTR"
		dig @$DNS $PTR -t APL +tcp | sed -n -f $SED_APL | sh >> $TMPFILE
	done
done

echo ; echo "Completed getting all the lists"

# Copy the results to their final location
BCNT=0
BCNT=`sed -n '$=' $BLOCK_TMP`
if [ $BLOCK_LIST ] && [ $BCNT -gt 0 ]
then
	BCNT=`expr $BCNT - 1`
	echo "Adding $BCNT blocked addresses"
	cp $BLOCK_TMP $BLOCK_FILE
fi

ACNT=0
ACNT=`sed -n '$=' $ALLOW_TMP`
if [ $ALLOW_LIST ] && [ $ACNT -gt 0 ]
then
	ACNT=`expr $ACNT - 1`
	echo "Adding $ACNT allowed addresses"
	cp $ALLOW_TMP $ALLOW_FILE
fi

# Delete the temporary files
rm $BLOCK_TMP $ALLOW_TMP

# Reload PF, if it is enabled in the configuration file.
if [ $RELOAD_PF == "YES" ]
then
	# If there is an allow list in use, we can reload both the
	# block and allow tables with one command.
	#if [ $ACNT -gt 0 ]
	if [ $ALLOW_LIST ]
	then
		PFCTL_TBL="-t $BLOCK_TABLE -t $ALLOW_TABLE"
	else
		PFCTL_TBL="-t $BLOCK_TABLE"
	fi
	
	echo -n "Reloading pf..."
	/sbin/pfctl -f /etc/pf.conf $PFCTL_TBL
	echo "done"
fi

# Calculate runtime
END_EPOCH=`date +%s`
SECONDS=$((END_EPOCH - START_EPOCH))
HOURS=$((SECONDS / 3600))
SECONDS=$((SECONDS % 3600))
MINUTES=$((SECONDS / 60))
SECONDS=$((SECONDS % 60))

echo ; echo "Finished ThreatSTOP update at `date`"
echo "Run Length: $HOURS hour(s) $MINUTES minute(s) $SECONDS second(s)"

#####################################################################
# Revision History
#
# ver 1.3 - 12/30/2010
# - Rewrote the script in sh, previous version was in bash
# - Rewrote part that gets the list
# - Improved handling of non-responsive DNS servers
# - Improved parsing of the DNS answers
# - More complete install script. Everything is copied to the correct place,
#   prompting the user to replace any existing files. Creates cron jobs to
#   run the script to update the lists and to send the log. Modify the pflog
#   entry in newsyslog.conf to rotate the log every night at 12:00 AM.
# - Logupload and sendlog scripts. Checks if pflog file is compressed or not,
#   and unzips it if is. Log uploading is now done with a Perl script that
#   uploads the log to the secure ThreatSTOP web server.
#
# ver 1.2 - 11/29/2010
# - Redid the PF reloading. The pfctl "-T load" option was removed
#
# ver 1.1 - 1/11/2010
# - Removed the arrays used to hold all the results. Caused the arrays to 
#   be huge and processing it took a lot of time. Now it dumps the results
#   immediatly after parsing the result.
# - Added text at end to show how long the script ran, in hours, minutes, 
#   and seconds. Easier to read than the start and end times alone.
# - install.sh script is a little smarter. Creates directories if needed.
# - Created the sendlog.sh script to email yesterdays pf log to ThreatSTOP.
# - Issue found: if a lookup fails, the outer loop that goes through the 
#   available DNS servers is started from the begining, redoing the lookups
#   that it already completed. It should try the next DNS server, and for
#   future lookups, without starting over. 
#
# ver 1.0 - 1/8/2010
# - Initial version
# - Port of the Perl Pf script
